(function (global, factory) {
  "object" === typeof exports ? module.exports = factory() :
  "function" === typeof define && define.amd ? define(factory) :
  global.History = factory();
}(this, function () {

  'use strict';

  // avoid requiring files if CJS or AMD is not supported
  var cjs = 'object' === typeof exports;
  var amd = 'function' === typeof define && define.amd;

  if(cjs || amd)
    var Happens = require('happens');

  /**
   * History class.
   * 
   * Handle all /push/state url's interaction with the browser.
   */
  function History(){

    Happens(this);

    this.history = window.history;

    var self = this;
    var popped = false;
    var initial = this.pathname();
    var hash = window.location.hash || null;

    if(hash && hash.length)
      this.replace(window.location.hash.substr(1));

    window.addEventListener('popstate', function(){
      // skips first pop if present (like in chrome)
      if(initial == self.pathname() && !popped)
        return popped = true;

      self.emit('url:change', window.location.pathname);
    }, false);
  };


  History.prototype.pathname = function(){
    return window.location.pathname;
  }

  History.prototype.push = function(url, title, state){
    window.history.pushState(state, title, url);
    if(title) document.title = title;
    this.emit('url:change', window.location.pathname);
  }

  History.prototype.replace = function(url, title, state){
    window.history.replaceState(state, title, url);
    if(title) document.title = title;
  };

  return History;
}));